<template>
  <div class="container">
    <h1>Child Survey Report</h1>
    <section class="row">
      <div class="col-6 survey-date-select">
        <label>Select Date:</label>
        <date-picker
          v-model="sessionDate"
          :config="datePickerOptions"
          @dp-show="clearRange"
          @dp-hide="getSessionId"
          name="date" />
      </div>
    </section>
    <section class="row">
      <div class="col-6">
        <p class="select-separator"> - or - </p>
        <label>Select Range:</label>
      </div>
    </section>
    <section class="row">
      <div class="col-3">
        <p>Start Date</p>
        <date-picker
          v-model="startDate"
          :config="datePickerOptions"
          @dp-show="clearSingle"
          name="date" />
      </div>
      <div class="col-3">
        <p>End Date</p>
        <date-picker
          v-model="endDate"
          :config="datePickerOptions"
          @dp-show="clearSingle"
          @dp-hide="getDataForDateRange"
          name="date" />
      </div>
    </section>
    <section class="row">
      <div
        v-if="surveyData.length"
        class="button-container col-6">
        <button
          class="btn btn-info buttons print-surveys"
          @click="printTable">Print Survey Results</button>
        <button
          class="btn btn-info buttons export-surveys"
          @click="exportTable('surveyResults_' + filename + '.csv')">Export Survey Results to .csv</button>
      </div>
    </section>
    <section
      v-if="surveyData.length"
      class="row survey-results-container">
      <main
        v-for="session in surveyData"
        :key="session.id"
        class="col-12">
        <table v-if="session.questions.length">
          <caption class="survey-school-name">School: {{ session.school }}</caption>
          <thead>
            <tr class="header-row">
              <th class="survey-table-header">Question</th>
              <th class="survey-table-header">Answers (response: number of total times given)</th>
            </tr>
          </thead>
          <tbody>
            <tr
              v-for="entry in session.questions"
              class="survey-row"
              :key="entry.id"
              >
              <td class="survey-data">{{ entry.text }}</td>
              <td class="survey-data">
                <p v-if="entry.type !== 'text_area'">
                  <span
                    v-for="(value, key, index) in entry.answers"
                    :key="index">{{ key }}: {{ value }}{{ index + 1 !== Object.keys(entry.answers).length ? " || " : "" }}
                  </span>
                </p>
                <p v-else>
                  <span
                    v-for="(key, value, index) in entry.answers"
                    :key="index">{{ value }}{{ index + 1 !== Object.keys(entry.answers).length ? ", " : "" }}
                  </span>
                </p>
              </td>
            </tr>
          </tbody>
        </table>
      </main>
    </section>
  </div>
</template>

<script>
export default {
  data() {
    return {
      datePickerOptions: {
        format: "YYYY-MM-DD",
        useCurrent: false,
        showClear: true,
        showClose: true
      },
      sessionDate: null,
      startDate: null,
      endDate: null,
      surveyData: []
    }
  },
  computed: {
    filename() {
      if (!this.sessionDate) {
        return this.startDate + "_" + this.endDate;
      }
      return this.sessionDate;
    }
  },
  methods: {
    getSessionId() {
      if(!this.sessionDate) {
        return;
      }
      app.api.Get("sessions/byDate/" + this.sessionDate).then(response => {
        if (response.length) {
          return this.getData(response[0].id);

        }
        this.clearSingle();
        return app.Message( "No session exists for the selected date.", "warn" );
      });
    },
    getData(id) {
      app.api.Get("survey/bySessionId/" + id).then(response => {
        if(response.length) {
          return this.surveyData = response;
        }
        this.clearSingle();
        return app.Message( "No survey data exists for the session held on this date.", "warn" );
      });
    },
    getDataForDateRange() {
      if (!this.endDate || !this.startDate) {
        return app.Message( "Please select both a start and end date.", "warn" );
      }
      let range = {
        start: this.startDate,
        end: this.endDate
      };
      let showData = false;
      app.api.Post("survey/byDateRange", range)
        .then(response => {
          if(response.length) {
            response.forEach(entry => {
              if(Object.keys(entry.questions).length) {
                showData = true;
              }
            })
            if(showData) {
              return this.surveyData = response;
            } else {
              return app.Message( "No survey data exists for any of the sessions held in this range.", "warn" );
            }
          }
          this.clearRange();
          return app.Message( "No sessions exist for the selected date range.", "warn" );
        });
    },
    clearRange() {
      if(this.startDate || this.endDate) {
        this.startDate = null;
        this.endDate = null;
      }
      if(this.surveyData) {
        this.surveyData = [];
      }
    },
    clearSingle() {
      if(this.sessionDate) {
        this.sessionDate = null;
      }
      if(this.surveyData) {
        this.surveyData = [];
      }
    },
    printTable() {
      const table = document.querySelector(".survey-results-container").cloneNode(true);
      //TODO: add styles for table
      const openedWindow = window.open("", "_blank", "height=1000,width=800");
      openedWindow.document.body.appendChild(table);
      openedWindow.window.print();
      openedWindow.window.close();
    },
    downloadCsv(csv, filename) {
      let csvFile = null;
      let downloadLink = "";
      csvFile = new Blob([csv], {type: "text/csv"});
      downloadLink = document.createElement("a");
      downloadLink.download = filename;
      downloadLink.href = window.URL.createObjectURL(csvFile);
      downloadLink.style.display = "none";
      document.body.appendChild(downloadLink);
      downloadLink.click();
    },
    exportTable(filename) {
      let csv = [];
      let rows = document.querySelectorAll("table tr");
      let schoolNameIndex = 0;
      for (let i = 0; i < rows.length; i++) {
        let schoolNames = document.getElementsByClassName("survey-school-name");
        let row = [], cols = rows[i].querySelectorAll("td, th");
        if(rows[i].className === "header-row") {
          row.push(schoolNames[schoolNameIndex].innerText.replace(',', '/'));
          schoolNameIndex += 1;
          csv.push(row);
          row = [];
        }
        for (let j = 0; j < cols.length; j++) {
          row.push(cols[j].innerText);
        }
        csv.push(row.join(","));
      }
      this.downloadCsv(csv.join("\n"), filename);
    }
  }
}
</script>

<style>
  .survey-date-select {
    padding: 10px 15px;
  }
  .button-container {
    padding: 10px 0;
  }
  .select-separator {
    text-align: center;
  }
  .survey-school-name {
    font-size: 18px;
    font-weight: 600;
    caption-side: top !important;
    padding: 10px 0;
  }
  .survey-table-header {
    width: 50%;
    font-size: 18px;
    padding-bottom: 10px;
    text-decoration: underline;
  }
  .survey-row {
    border-bottom: 1px solid;
  }
  .survey-data {
    padding: 10px 0;
  }
  @media print {
    .survey-row {
      border-bottom: 1px solid;
    }
  }

</style>